# $Id$

package MultiImageUploader::CMS;

use strict;

sub upload {
    my $app = shift;
    my (%opt) = @_;

    my $q = $app->{query};
    my $blog_id = scalar $q->param('blog_id');

    my $tmpl = $app->load_tmpl('upload.tmpl');
    $tmpl->param('blog_id'  => $blog_id);
    $app->add_breadcrumb($app->translate("Main Menu"),$app->{mtscript_url});
    require MT::Blog;
    my $blog = MT::Blog->load ($blog_id);
    $app->add_breadcrumb($blog->name,$app->mt_uri( mode => 'menu', args => { blog_id => $blog_id }));
    $app->add_breadcrumb($app->translate("MultiImageUploader"));

    $tmpl->param(breadcrumbs       => $app->{breadcrumbs});
    return $app->build_page($tmpl);
}

sub doLog {
    my ($msg) = @_; 
    return unless defined($msg);

    require MT::Log;
    my $log = MT::Log->new;
    $log->message($msg) ;
    $log->save or die $log->errstr;
}

sub save {
    my $app = shift;
    my (%opt) = @_;

    my $q = $app->{query};
    my $blog_id = scalar $q->param('blog_id');
    my $blog = MT::Blog->load($blog_id);
    my $fmgr = $blog->file_mgr;
    my $author = $app->user;
    my $author_id = $author->id;

    my $thumb_size = $q->param('thumb_size') || 100;
    my $thumb_dir = $q->param('thumb_dir');
    my $page_title = $q->param('page_title');
    my $make_thumb = $q->param('thumb_make');

    my $html;
    my $img_cnt = 0;
    my @fhs = $q->upload('file1[]');
    @fhs = reverse(@fhs);
    foreach my $fh (@fhs) {
        #filename
        my $fname = $fh;

        #extension
        $fname =~ m!(.*?)\.(.*)$!;
        my $base = $1;
        my $ext = $2;
        if(!$base) {
            $base = $fname;
            $ext = '';
        }

        #mimetype
        my $mimetype = $q->uploadInfo($fh)->{'Content-Type'};

        #path
        my $plugin = MT::Plugin::MultiImageUploader->instance();
        my $path =  $plugin->multi_image_uploader_dir($blog_id) || 'upload_files';
        my $directory = $path;
        
        my $root_path = $blog->site_path;
        $root_path = File::Spec->canonpath($root_path);
        my $dest_path = File::Spec->catfile($root_path, $path);
        my $dest_file = File::Spec->catfile($dest_path, $fname);
        
        my $i;
        while (-e $dest_file) {
            $i++;
            $fname = $base . '_' . $i;
            $fname .= ".$ext" if $ext;
            $dest_file = File::Spec->catfile($dest_path, $fname);
        }
        my $root_url = $blog->site_url;
        my $url = $root_url;
        $url .= '/' if $url !~ m!/$!;
        $url .= "$path/" if $path;
        $url .= $fname;

        unless ($fmgr->exists($dest_path)) {
            $fmgr->mkpath($dest_path)
                or return $app->error(
                    $app->translate(
                        "Can't make path '[_1]': [_2]",
                        $dest_path,
                        $fmgr->errstr
                    )
                );
        }

        my $umask = oct $app->config('UploadUmask');
        my $old = umask($umask);
        defined(my $bytes = $fmgr->put($fh, $dest_file, 'upload'))
            or return $app->error(
                $app->translate(
                    "Error writing upload to '[_1]': [_2]",
                    $dest_file,
                    $fmgr->errstr
                )
            );
        umask($old);

        #----- make asset
        my $local_file = $dest_file;
        eval { require Image::Size; };
        die 'Perl module Image::Size is required to determine width and height of uploaded images.' if $@;
        my ( $w, $h, $id ) = Image::Size::imgsize($local_file);
    
        require File::Basename;
        my $local_basename = File::Basename::basename($fname);
        my $local_ext = ( File::Basename::fileparse( $fname, qr/[A-Za-z0-9]+$/ ) )[2];

        require MT::Asset;
        my $asset_pkg = MT::Asset->handler_for_file($local_basename);
        my $is_image = 0;
        if ( defined($w) && defined($h) ) {
            $is_image = 1
                if $asset_pkg->isa('MT::Asset::Image');
        }
        else {
            # rebless to file type
            $asset_pkg = 'MT::Asset';
        }
        my $asset;
        if (!($asset = $asset_pkg->load(
                    { file_path => $local_file, blog_id => $blog_id })))
        {
            $asset = $asset_pkg->new();
            $asset->file_path($local_file);
            $asset->file_name($local_basename);
            $asset->file_ext($local_ext);
            $asset->blog_id($blog_id);
            $asset->created_by( $author->id );
        }
        else {
            $asset->modified_by( $author->id );
        }
        my $original = $asset->clone;
        $asset->url($url);
        if ($is_image) {
            $asset->image_width($w);
            $asset->image_height($h);
        }
        $asset->mime_type($mimetype);
        use MT::Tag;
        $asset->save;
    
        $blog->touch('asset');
        $blog->save;

        $img_cnt++;
        my $text;
        if ($make_thumb) {
            #----- WebPage
            $text .= "<img src=\"$url\" width=\"$w\" height=\"$h\" alt=\"$base\" title=\"$base\" />\n";

            my $title;
            if ($page_title) {
                $title = $page_title . sprintf("%02d", $img_cnt);
            } else {
                $title = $base;
            }
            
            use MT::Page;
            my $page = MT::Page->new;
            $page->blog_id($blog_id);
            $page->author_id($author_id);
            $page->title($title);

            $local_basename =~ m!(.*?)\.(.*)$!;
            my $f = $1;
            if(!$f) {
                $f = $local_basename;
            }
            $page->basename($f);

            $page->text($text);
            $page->status(MT::Entry::RELEASE());
            use MT::Tag;
            $page->save
                or die $page->errstr;
    
            my $mt = MT->instance;
            $mt->rebuild_entry( Entry => $page, Blog => $blog, BuildDependencies => 1 );
            
            #----- create a thumbnail
            require MT::Image;
            my $img = new MT::Image( Filename => $local_file )
              or return $asset->error( MT::Image->errstr );
    
            my $data;
            if ($thumb_dir eq 'w') {
                $data = $img->scale( Width => $thumb_size )
            } else {
                $data = $img->scale( Height => $thumb_size )
            }
    
            $data or return $asset->error(MT->translate( "Error scaling image: [_1]", $img->errstr ) );
    
            my $thumb_fname = $base . '_' . $thumb_size;
            $thumb_fname .= ".$ext" if $ext;
    
            my $thumbnail = File::Spec->catfile($dest_path, $thumb_fname);
    
            $fmgr->put_data( $data, $thumbnail, 'upload' );
    
            my $thumb_url = $root_url;
            $thumb_url .= '/' if $thumb_url !~ m!/$!;
            $thumb_url .= "$path/" if $path;
            $thumb_url .= $thumb_fname;
    
            #----- create gallery HTML
    
            my $nw = $img->{width};
            my $nh = $img->{height};
            my $permalink = $page->permalink;
    
            $html .= "<a href=\"$permalink\" title=\"$base\" target=\"_blank\"><img src=\"$thumb_url\" width=\"$nw\" height=\"$nh\" alt=\"$base\" title=\"$base\" /></a>\n";
        } else {
            $html .= "<img src=\"$url\" width=\"$w\" height=\"$h\" alt=\"$base\" title=\"$base\" />\n";
        }
    }

    my $tmpl = $app->load_tmpl('complete.tmpl');
    $tmpl->param('blog_id'  => $blog_id);
    $app->add_breadcrumb($app->translate("Main Menu"),$app->{mtscript_url});
    require MT::Blog;
    my $blog = MT::Blog->load ($blog_id);
    $app->add_breadcrumb($blog->name,$app->mt_uri( mode => 'menu', args => { blog_id => $blog_id }));
    $app->add_breadcrumb($app->translate("MultiImageUploader"));

    $tmpl->param(breadcrumbs => $app->{breadcrumbs});
    
    $tmpl->param(gallery_code => $html);

    return $app->build_page($tmpl);
}

1;